@@ -13,6 +13,13 @@ extern "C" {
 
 #include "../cglm.h"
 
+/* DEPRECATED! use _diag */
+#define glmc_aabb2d_size(aabb) glmc_aabb2d_diag(aabb)
+
+CGLM_EXPORT
+void
+glmc_aabb2d_zero(vec2 aabb[2]);
+
 CGLM_EXPORT
 void
 glmc_aabb2d_copy(vec2 aabb[2], vec2 dest[2]);
@@ -46,7 +53,11 @@ glmc_aabb2d_isvalid(vec2 aabb[2]);
 
 CGLM_EXPORT
 float
-glmc_aabb2d_size(vec2 aabb[2]);
+glmc_aabb2d_diag(vec2 aabb[2]);
+
+CGLM_EXPORT
+void
+glmc_aabb2d_sizev(vec2 aabb[2], vec2 dest);
 
 CGLM_EXPORT
 float
