@@ -23,15 +23,15 @@ glmc_mat2x3_zero(mat2x3 mat);
 
 CGLM_EXPORT
 void
-glmc_mat2x3_make(float * __restrict src, mat2x3 dest);
+glmc_mat2x3_make(const float * __restrict src, mat2x3 dest);
 
 CGLM_EXPORT
 void
-glmc_mat2x3_mul(mat2x3 m1, mat3x2 m2, mat2 dest);
+glmc_mat2x3_mul(mat2x3 m1, mat3x2 m2, mat3 dest);
 
 CGLM_EXPORT
 void
-glmc_mat2x3_mulv(mat2x3 m, vec3 v, vec2 dest);
+glmc_mat2x3_mulv(mat2x3 m, vec2 v, vec3 dest);
 
 CGLM_EXPORT
 void
