@@ -44,6 +44,7 @@
    CGLM_INLINE mat4s   glms_quat_rotate(mat4s m, versors q)
    CGLM_INLINE mat4s   glms_quat_rotate_at(mat4s m, versors q, vec3s pivot)
    CGLM_INLINE mat4s   glms_quat_rotate_atm(versors q, vec3s pivot)
+   CGLM_INLINE versors glms_quat_make(float * restrict src)
  */
 
 #ifndef cglms_quat_h
@@ -54,6 +55,9 @@
 #include "../plane.h"
 #include "../quat.h"
 
+/* api definition */
+#define glms_quat_(NAME) CGLM_STRUCTAPI(quat, NAME)
+
 /*
  * IMPORTANT:
  * ----------------------------------------------------------------------------
@@ -74,7 +78,7 @@
  */
 CGLM_INLINE
 versors
-glms_quat_identity(void) {
+glms_quat_(identity)(void) {
   versors dest;
   glm_quat_identity(dest.raw);
   return dest;
@@ -90,7 +94,7 @@ glms_quat_identity(void) {
  */
 CGLM_INLINE
 void
-glms_quat_identity_array(versors * __restrict q, size_t count) {
+glms_quat_(identity_array)(versors * __restrict q, size_t count) {
   CGLM_ALIGN(16) versor v = GLM_QUAT_IDENTITY_INIT;
   size_t i;
 
@@ -110,7 +114,7 @@ glms_quat_identity_array(versors * __restrict q, size_t count) {
  */
 CGLM_INLINE
 versors
-glms_quat_init(float x, float y, float z, float w) {
+glms_quat_(init)(float x, float y, float z, float w) {
   versors dest;
   glm_quat_init(dest.raw, x, y, z, w);
   return dest;
@@ -157,7 +161,7 @@ glms_quat(float angle, float x, float y, float z) {
  */
 CGLM_INLINE
 versors
-glms_quat_from_vecs(vec3s a, vec3s b) {
+glms_quat_(from_vecs)(vec3s a, vec3s b) {
   versors dest;
   glm_quat_from_vecs(a.raw, b.raw, dest.raw);
   return dest;
@@ -170,7 +174,7 @@ glms_quat_from_vecs(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 float
-glms_quat_norm(versors q) {
+glms_quat_(norm)(versors q) {
   return glm_quat_norm(q.raw);
 }
 
@@ -182,7 +186,7 @@ glms_quat_norm(versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_normalize(versors q) {
+glms_quat_(normalize)(versors q) {
   versors dest;
   glm_quat_normalize_to(q.raw, dest.raw);
   return dest;
@@ -197,7 +201,7 @@ glms_quat_normalize(versors q) {
  */
 CGLM_INLINE
 float
-glms_quat_dot(versors p, versors q) {
+glms_quat_(dot)(versors p, versors q) {
   return glm_quat_dot(p.raw, q.raw);
 }
 
@@ -209,7 +213,7 @@ glms_quat_dot(versors p, versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_conjugate(versors q) {
+glms_quat_(conjugate)(versors q) {
   versors dest;
   glm_quat_conjugate(q.raw, dest.raw);
   return dest;
@@ -223,7 +227,7 @@ glms_quat_conjugate(versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_inv(versors q) {
+glms_quat_(inv)(versors q) {
   versors dest;
   glm_quat_inv(q.raw, dest.raw);
   return dest;
@@ -238,7 +242,7 @@ glms_quat_inv(versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_add(versors p, versors q) {
+glms_quat_(add)(versors p, versors q) {
   versors dest;
   glm_quat_add(p.raw, q.raw, dest.raw);
   return dest;
@@ -253,7 +257,7 @@ glms_quat_add(versors p, versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_sub(versors p, versors q) {
+glms_quat_(sub)(versors p, versors q) {
   versors dest;
   glm_quat_sub(p.raw, q.raw, dest.raw);
   return dest;
@@ -266,7 +270,7 @@ glms_quat_sub(versors p, versors q) {
  */
 CGLM_INLINE
 vec3s
-glms_quat_imagn(versors q) {
+glms_quat_(imagn)(versors q) {
   vec3s dest;
   glm_normalize_to(q.raw, dest.raw);
   return dest;
@@ -279,7 +283,7 @@ glms_quat_imagn(versors q) {
  */
 CGLM_INLINE
 float
-glms_quat_imaglen(versors q) {
+glms_quat_(imaglen)(versors q) {
   return glm_quat_imaglen(q.raw);
 }
 
@@ -290,7 +294,7 @@ glms_quat_imaglen(versors q) {
  */
 CGLM_INLINE
 float
-glms_quat_angle(versors q) {
+glms_quat_(angle)(versors q) {
   return glm_quat_angle(q.raw);
 }
 
@@ -302,7 +306,7 @@ glms_quat_angle(versors q) {
  */
 CGLM_INLINE
 vec3s
-glms_quat_axis(versors q) {
+glms_quat_(axis)(versors q) {
   vec3s dest;
   glm_quat_axis(q.raw, dest.raw);
   return dest;
@@ -322,7 +326,7 @@ glms_quat_axis(versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_mul(versors p, versors q) {
+glms_quat_(mul)(versors p, versors q) {
   versors dest;
   glm_quat_mul(p.raw, q.raw, dest.raw);
   return dest;
@@ -336,7 +340,7 @@ glms_quat_mul(versors p, versors q) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_mat4(versors q) {
+glms_quat_(mat4)(versors q) {
   mat4s dest;
   glm_quat_mat4(q.raw, dest.raw);
   return dest;
@@ -350,7 +354,7 @@ glms_quat_mat4(versors q) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_mat4t(versors q) {
+glms_quat_(mat4t)(versors q) {
   mat4s dest;
   glm_quat_mat4t(q.raw, dest.raw);
   return dest;
@@ -364,7 +368,7 @@ glms_quat_mat4t(versors q) {
  */
 CGLM_INLINE
 mat3s
-glms_quat_mat3(versors q) {
+glms_quat_(mat3)(versors q) {
   mat3s dest;
   glm_quat_mat3(q.raw, dest.raw);
   return dest;
@@ -378,7 +382,7 @@ glms_quat_mat3(versors q) {
  */
 CGLM_INLINE
 mat3s
-glms_quat_mat3t(versors q) {
+glms_quat_(mat3t)(versors q) {
   mat3s dest;
   glm_quat_mat3t(q.raw, dest.raw);
   return dest;
@@ -395,7 +399,7 @@ glms_quat_mat3t(versors q) {
  */
 CGLM_INLINE
 versors
-glms_quat_lerp(versors from, versors to, float t) {
+glms_quat_(lerp)(versors from, versors to, float t) {
   versors dest;
   glm_quat_lerp(from.raw, to.raw, t, dest.raw);
   return dest;
@@ -412,7 +416,7 @@ glms_quat_lerp(versors from, versors to, float t) {
  */
 CGLM_INLINE
 versors
-glms_quat_lerpc(versors from, versors to, float t) {
+glms_quat_(lerpc)(versors from, versors to, float t) {
   versors dest;
   glm_quat_lerpc(from.raw, to.raw, t, dest.raw);
   return dest;
@@ -430,7 +434,7 @@ glms_quat_lerpc(versors from, versors to, float t) {
  */
 CGLM_INLINE
 versors
-glms_quat_nlerp(versors from, versors to, float t) {
+glms_quat_(nlerp)(versors from, versors to, float t) {
   versors dest;
   glm_quat_nlerp(from.raw, to.raw, t, dest.raw);
   return dest;
@@ -442,12 +446,12 @@ glms_quat_nlerp(versors from, versors to, float t) {
  *
  * @param[in]   from  from
  * @param[in]   to    to
- * @param[in]   t     amout
+ * @param[in]   t     amount
  * @returns result quaternion
  */
 CGLM_INLINE
 versors
-glms_quat_slerp(versors from, versors to, float t) {
+glms_quat_(slerp)(versors from, versors to, float t) {
   versors dest;
   glm_quat_slerp(from.raw, to.raw, t, dest.raw);
   return dest;
@@ -462,7 +466,7 @@ glms_quat_slerp(versors from, versors to, float t) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_look(vec3s eye, versors ori) {
+glms_quat_(look)(vec3s eye, versors ori) {
   mat4s dest;
   glm_quat_look(eye.raw, ori.raw, dest.raw);
   return dest;
@@ -477,7 +481,7 @@ glms_quat_look(vec3s eye, versors ori) {
  */
 CGLM_INLINE
 versors
-glms_quat_for(vec3s dir, vec3s up) {
+glms_quat_(for)(vec3s dir, vec3s up) {
   versors dest;
   glm_quat_for(dir.raw, up.raw, dest.raw);
   return dest;
@@ -494,7 +498,7 @@ glms_quat_for(vec3s dir, vec3s up) {
  */
 CGLM_INLINE
 versors
-glms_quat_forp(vec3s from, vec3s to, vec3s up) {
+glms_quat_(forp)(vec3s from, vec3s to, vec3s up) {
   versors dest;
   glm_quat_forp(from.raw, to.raw, up.raw, dest.raw);
   return dest;
@@ -509,7 +513,7 @@ glms_quat_forp(vec3s from, vec3s to, vec3s up) {
  */
 CGLM_INLINE
 vec3s
-glms_quat_rotatev(versors q, vec3s v) {
+glms_quat_(rotatev)(versors q, vec3s v) {
   vec3s dest;
   glm_quat_rotatev(q.raw, v.raw, dest.raw);
   return dest;
@@ -524,7 +528,7 @@ glms_quat_rotatev(versors q, vec3s v) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_rotate(mat4s m, versors q) {
+glms_quat_(rotate)(mat4s m, versors q) {
   glm_quat_rotate(m.raw, q.raw, m.raw);
   return m;
 }
@@ -538,7 +542,7 @@ glms_quat_rotate(mat4s m, versors q) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_rotate_at(mat4s m, versors q, vec3s pivot) {
+glms_quat_(rotate_at)(mat4s m, versors q, vec3s pivot) {
   glm_quat_rotate_at(m.raw, q.raw, pivot.raw);
   return m;
 }
@@ -556,10 +560,24 @@ glms_quat_rotate_at(mat4s m, versors q, vec3s pivot) {
  */
 CGLM_INLINE
 mat4s
-glms_quat_rotate_atm(versors q, vec3s pivot) {
+glms_quat_(rotate_atm)(versors q, vec3s pivot) {
   mat4s dest;
   glm_quat_rotate_atm(dest.raw, q.raw, pivot.raw);
   return dest;
 }
 
+/*!
+ * @brief Create CGLM quaternion from pointer
+ *
+ * @param[in]  src  pointer to an array of floats
+ * @returns constructed quaternion from raw pointer
+ */
+CGLM_INLINE
+versors
+glms_quat_(make)(const float * __restrict src) {
+  versors dest;
+  glm_quat_make(src, dest.raw);
+  return dest;
+}
+
 #endif /* cglms_quat_h */
