@@ -41,9 +41,14 @@
    CGLM_INLINE vec3s glms_vec3_muladds(vec3s a, float s, vec3s dest);
    CGLM_INLINE vec3s glms_vec3_maxadd(vec3s a, vec3s b, vec3s dest);
    CGLM_INLINE vec3s glms_vec3_minadd(vec3s a, vec3s b, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_subsub(vec3s a, vec3s b, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_addsub(vec3s a, vec3s b, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_mulsub(vec3s a, vec3s b, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_mulsubs(vec3s a, float s, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_maxsub(vec3s a, vec3s b, vec3s dest);
+   CGLM_INLINE vec3s glms_vec3_minsub(vec3s a, vec3s b, vec3s dest);
    CGLM_INLINE vec3s glms_vec3_flipsign(vec3s v);
    CGLM_INLINE vec3s glms_vec3_negate(vec3s v);
-   CGLM_INLINE vec3s glms_vec3_inv(vec3s v);
    CGLM_INLINE vec3s glms_vec3_normalize(vec3s v);
    CGLM_INLINE vec3s glms_vec3_cross(vec3s a, vec3s b);
    CGLM_INLINE vec3s glms_vec3_crossn(vec3s a, vec3s b);
@@ -70,6 +75,10 @@
    CGLM_INLINE vec3s glms_vec3_smoothinterp(vec3s from, vec3s to, float t);
    CGLM_INLINE vec3s glms_vec3_smoothinterpc(vec3s from, vec3s to, float t);
    CGLM_INLINE vec3s glms_vec3_swizzle(vec3s v, int mask);
+   CGLM_INLINE vec3s glms_vec3_make(float * restrict src);
+   CGLM_INLINE vec3s glms_vec3_faceforward(vec3s n, vec3s v, vec3s nref);
+   CGLM_INLINE vec3s glms_vec3_reflect(vec3s v, vec3s n);
+   CGLM_INLINE bool  glms_vec3_refract(vec3s v, vec3s n, float eta, vec3s *dest)
 
  Convenient:
    CGLM_INLINE vec3s glms_cross(vec3s a, vec3s b);
@@ -119,7 +128,7 @@ glms_vec3(vec4s v4) {
  */
 CGLM_INLINE
 void
-glms_vec3_pack(vec3s dst[], vec3 src[], size_t len) {
+glms_vec3_(pack)(vec3s dst[], vec3 src[], size_t len) {
   size_t i;
 
   for (i = 0; i < len; i++) {
@@ -136,7 +145,7 @@ glms_vec3_pack(vec3s dst[], vec3 src[], size_t len) {
  */
 CGLM_INLINE
 void
-glms_vec3_unpack(vec3 dst[], vec3s src[], size_t len) {
+glms_vec3_(unpack)(vec3 dst[], vec3s src[], size_t len) {
   size_t i;
 
   for (i = 0; i < len; i++) {
@@ -151,7 +160,7 @@ glms_vec3_unpack(vec3 dst[], vec3s src[], size_t len) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_zero(void) {
+glms_vec3_(zero)(void) {
   vec3s r;
   glm_vec3_zero(r.raw);
   return r;
@@ -164,7 +173,7 @@ glms_vec3_zero(void) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_one(void) {
+glms_vec3_(one)(void) {
   vec3s r;
   glm_vec3_one(r.raw);
   return r;
@@ -180,7 +189,7 @@ glms_vec3_one(void) {
  */
 CGLM_INLINE
 float
-glms_vec3_dot(vec3s a, vec3s b) {
+glms_vec3_(dot)(vec3s a, vec3s b) {
   return glm_vec3_dot(a.raw, b.raw);
 }
 
@@ -188,7 +197,7 @@ glms_vec3_dot(vec3s a, vec3s b) {
  * @brief norm * norm (magnitude) of vec
  *
  * we can use this func instead of calling norm * norm, because it would call
- * sqrtf fuction twice but with this func we can avoid func call, maybe this is
+ * sqrtf function twice but with this func we can avoid func call, maybe this is
  * not good name for this func
  *
  * @param[in] v vector
@@ -197,7 +206,7 @@ glms_vec3_dot(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 float
-glms_vec3_norm2(vec3s v) {
+glms_vec3_(norm2)(vec3s v) {
   return glm_vec3_norm2(v.raw);
 }
 
@@ -210,7 +219,7 @@ glms_vec3_norm2(vec3s v) {
  */
 CGLM_INLINE
 float
-glms_vec3_norm(vec3s v) {
+glms_vec3_(norm)(vec3s v) {
   return glm_vec3_norm(v.raw);
 }
 
@@ -230,7 +239,7 @@ glms_vec3_norm(vec3s v) {
  */
 CGLM_INLINE
 float
-glms_vec3_norm_one(vec3s v) {
+glms_vec3_(norm_one)(vec3s v) {
   return glm_vec3_norm_one(v.raw);
 }
 
@@ -249,7 +258,7 @@ glms_vec3_norm_one(vec3s v) {
  */
 CGLM_INLINE
 float
-glms_vec3_norm_inf(vec3s v) {
+glms_vec3_(norm_inf)(vec3s v) {
   return glm_vec3_norm_inf(v.raw);
 }
 
@@ -262,7 +271,7 @@ glms_vec3_norm_inf(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_add(vec3s a, vec3s b) {
+glms_vec3_(add)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_add(a.raw, b.raw, r.raw);
   return r;
@@ -277,7 +286,7 @@ glms_vec3_add(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_adds(vec3s a, float s) {
+glms_vec3_(adds)(vec3s a, float s) {
   vec3s r;
   glm_vec3_adds(a.raw, s, r.raw);
   return r;
@@ -292,7 +301,7 @@ glms_vec3_adds(vec3s a, float s) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_sub(vec3s a, vec3s b) {
+glms_vec3_(sub)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_sub(a.raw, b.raw, r.raw);
   return r;
@@ -307,14 +316,14 @@ glms_vec3_sub(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_subs(vec3s a, float s) {
+glms_vec3_(subs)(vec3s a, float s) {
   vec3s r;
   glm_vec3_subs(a.raw, s, r.raw);
   return r;
 }
 
 /*!
- * @brief multiply two vector (component-wise multiplication)
+ * @brief multiply two vectors (component-wise multiplication)
  *
  * @param     a     vector1
  * @param     b     vector2
@@ -322,7 +331,7 @@ glms_vec3_subs(vec3s a, float s) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_mul(vec3s a, vec3s b) {
+glms_vec3_(mul)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_mul(a.raw, b.raw, r.raw);
   return r;
@@ -337,7 +346,7 @@ glms_vec3_mul(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_scale(vec3s v, float s) {
+glms_vec3_(scale)(vec3s v, float s) {
   vec3s r;
   glm_vec3_scale(v.raw, s, r.raw);
   return r;
@@ -352,7 +361,7 @@ glms_vec3_scale(vec3s v, float s) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_scale_as(vec3s v, float s) {
+glms_vec3_(scale_as)(vec3s v, float s) {
   vec3s r;
   glm_vec3_scale_as(v.raw, s, r.raw);
   return r;
@@ -367,7 +376,7 @@ glms_vec3_scale_as(vec3s v, float s) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_div(vec3s a, vec3s b) {
+glms_vec3_(div)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_div(a.raw, b.raw, r.raw);
   return r;
@@ -382,7 +391,7 @@ glms_vec3_div(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_divs(vec3s a, float s) {
+glms_vec3_(divs)(vec3s a, float s) {
   vec3s r;
   glm_vec3_divs(a.raw, s, r.raw);
   return r;
@@ -399,7 +408,7 @@ glms_vec3_divs(vec3s a, float s) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_addadd(vec3s a, vec3s b, vec3s dest) {
+glms_vec3_(addadd)(vec3s a, vec3s b, vec3s dest) {
   glm_vec3_addadd(a.raw, b.raw, dest.raw);
   return dest;
 }
@@ -415,7 +424,7 @@ glms_vec3_addadd(vec3s a, vec3s b, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_subadd(vec3s a, vec3s b, vec3s dest) {
+glms_vec3_(subadd)(vec3s a, vec3s b, vec3s dest) {
   glm_vec3_subadd(a.raw, b.raw, dest.raw);
   return dest;
 }
@@ -431,7 +440,7 @@ glms_vec3_subadd(vec3s a, vec3s b, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_muladd(vec3s a, vec3s b, vec3s dest) {
+glms_vec3_(muladd)(vec3s a, vec3s b, vec3s dest) {
   glm_vec3_muladd(a.raw, b.raw, dest.raw);
   return dest;
 }
@@ -447,13 +456,13 @@ glms_vec3_muladd(vec3s a, vec3s b, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_muladds(vec3s a, float s, vec3s dest) {
+glms_vec3_(muladds)(vec3s a, float s, vec3s dest) {
   glm_vec3_muladds(a.raw, s, dest.raw);
   return dest;
 }
 
 /*!
- * @brief add max of two vector to result/dest
+ * @brief add max of two vectors to result/dest
  *
  * it applies += operator so dest must be initialized
  *
@@ -463,13 +472,13 @@ glms_vec3_muladds(vec3s a, float s, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_maxadd(vec3s a, vec3s b, vec3s dest) {
+glms_vec3_(maxadd)(vec3s a, vec3s b, vec3s dest) {
   glm_vec3_maxadd(a.raw, b.raw, dest.raw);
   return dest;
 }
 
 /*!
- * @brief add min of two vector to result/dest
+ * @brief add min of two vectors to result/dest
  *
  * it applies += operator so dest must be initialized
  *
@@ -479,11 +488,107 @@ glms_vec3_maxadd(vec3s a, vec3s b, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_minadd(vec3s a, vec3s b, vec3s dest) {
+glms_vec3_(minadd)(vec3s a, vec3s b, vec3s dest) {
   glm_vec3_minadd(a.raw, b.raw, dest.raw);
   return dest;
 }
 
+/*!
+ * @brief sub two vectors and sub result to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector 1
+ * @param[in]  b    vector 2
+ * @returns         dest -= (a - b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(subsub)(vec3s a, vec3s b, vec3s dest) {
+  glm_vec3_subsub(a.raw, b.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief add two vectors and sub result to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector 1
+ * @param[in]  b    vector 2
+ * @returns         dest -= (a + b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(addsub)(vec3s a, vec3s b, vec3s dest) {
+  glm_vec3_addsub(a.raw, b.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief mul two vectors and sub result to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector 1
+ * @param[in]  b    vector 2
+ * @returns         dest -= (a * b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(mulsub)(vec3s a, vec3s b, vec3s dest) {
+  glm_vec3_mulsub(a.raw, b.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief mul vector with scalar and sub result to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector
+ * @param[in]  s    scalar
+ * @returns         dest -= (a * b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(mulsubs)(vec3s a, float s, vec3s dest) {
+  glm_vec3_mulsubs(a.raw, s, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief sub max of two vectors to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector 1
+ * @param[in]  b    vector 2
+ * @returns         dest -= max(a, b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(maxsub)(vec3s a, vec3s b, vec3s dest) {
+  glm_vec3_maxsub(a.raw, b.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief sub min of two vectors to dest
+ *
+ * it applies -= operator so dest must be initialized
+ *
+ * @param[in]  a    vector 1
+ * @param[in]  b    vector 2
+ * @returns         dest -= min(a, b)
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(minsub)(vec3s a, vec3s b, vec3s dest) {
+  glm_vec3_minsub(a.raw, b.raw, dest.raw);
+  return dest;
+}
+
 /*!
  * @brief negate vector components and store result in dest
  *
@@ -492,7 +597,7 @@ glms_vec3_minadd(vec3s a, vec3s b, vec3s dest) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_flipsign(vec3s v) {
+glms_vec3_(flipsign)(vec3s v) {
   glm_vec3_flipsign(v.raw);
   return v;
 }
@@ -505,7 +610,7 @@ glms_vec3_flipsign(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_negate(vec3s v) {
+glms_vec3_(negate)(vec3s v) {
   glm_vec3_negate(v.raw);
   return v;
 }
@@ -518,7 +623,7 @@ glms_vec3_negate(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_normalize(vec3s v) {
+glms_vec3_(normalize)(vec3s v) {
   glm_vec3_normalize(v.raw);
   return v;
 }
@@ -532,7 +637,7 @@ glms_vec3_normalize(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_cross(vec3s a, vec3s b) {
+glms_vec3_(cross)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_cross(a.raw, b.raw, r.raw);
   return r;
@@ -547,14 +652,14 @@ glms_vec3_cross(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_crossn(vec3s a, vec3s b) {
+glms_vec3_(crossn)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_crossn(a.raw, b.raw, r.raw);
   return r;
 }
 
 /*!
- * @brief angle betwen two vector
+ * @brief angle between two vector
  *
  * @param[in] a  vector1
  * @param[in] b  vector2
@@ -563,7 +668,7 @@ glms_vec3_crossn(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 float
-glms_vec3_angle(vec3s a, vec3s b) {
+glms_vec3_(angle)(vec3s a, vec3s b) {
   return glm_vec3_angle(a.raw, b.raw);
 }
 
@@ -577,7 +682,7 @@ glms_vec3_angle(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_rotate(vec3s v, float angle, vec3s axis) {
+glms_vec3_(rotate)(vec3s v, float angle, vec3s axis) {
   glm_vec3_rotate(v.raw, angle, axis.raw);
   return v;
 }
@@ -597,7 +702,7 @@ glms_vec3_rotate(vec3s v, float angle, vec3s axis) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_rotate_m4(mat4s m, vec3s v) {
+glms_vec3_(rotate_m4)(mat4s m, vec3s v) {
   vec3s r;
   glm_vec3_rotate_m4(m.raw, v.raw, r.raw);
   return r;
@@ -612,7 +717,7 @@ glms_vec3_rotate_m4(mat4s m, vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_rotate_m3(mat3s m, vec3s v) {
+glms_vec3_(rotate_m3)(mat3s m, vec3s v) {
   vec3s r;
   glm_vec3_rotate_m3(m.raw, v.raw, r.raw);
   return r;
@@ -627,7 +732,7 @@ glms_vec3_rotate_m3(mat3s m, vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_proj(vec3s a, vec3s b) {
+glms_vec3_(proj)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_proj(a.raw, b.raw, r.raw);
   return r;
@@ -642,7 +747,7 @@ glms_vec3_proj(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_center(vec3s a, vec3s b) {
+glms_vec3_(center)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_center(a.raw, b.raw, r.raw);
   return r;
@@ -657,7 +762,7 @@ glms_vec3_center(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 float
-glms_vec3_distance(vec3s a, vec3s b) {
+glms_vec3_(distance)(vec3s a, vec3s b) {
   return glm_vec3_distance(a.raw, b.raw);
 }
 
@@ -670,7 +775,7 @@ glms_vec3_distance(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 float
-glms_vec3_distance2(vec3s a, vec3s b) {
+glms_vec3_(distance2)(vec3s a, vec3s b) {
   return glm_vec3_distance2(a.raw, b.raw);
 }
 
@@ -683,7 +788,7 @@ glms_vec3_distance2(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_maxv(vec3s a, vec3s b) {
+glms_vec3_(maxv)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_maxv(a.raw, b.raw, r.raw);
   return r;
@@ -698,7 +803,7 @@ glms_vec3_maxv(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_minv(vec3s a, vec3s b) {
+glms_vec3_(minv)(vec3s a, vec3s b) {
   vec3s r;
   glm_vec3_minv(a.raw, b.raw, r.raw);
   return r;
@@ -712,7 +817,7 @@ glms_vec3_minv(vec3s a, vec3s b) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_ortho(vec3s v) {
+glms_vec3_(ortho)(vec3s v) {
   vec3s r;
   glm_vec3_ortho(v.raw, r.raw);
   return r;
@@ -728,7 +833,7 @@ glms_vec3_ortho(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_clamp(vec3s v, float minVal, float maxVal) {
+glms_vec3_(clamp)(vec3s v, float minVal, float maxVal) {
   glm_vec3_clamp(v.raw, minVal, maxVal);
   return v;
 }
@@ -745,7 +850,7 @@ glms_vec3_clamp(vec3s v, float minVal, float maxVal) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_lerp(vec3s from, vec3s to, float t) {
+glms_vec3_(lerp)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_lerp(from.raw, to.raw, t, r.raw);
   return r;
@@ -763,7 +868,7 @@ glms_vec3_lerp(vec3s from, vec3s to, float t) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_lerpc(vec3s from, vec3s to, float t) {
+glms_vec3_(lerpc)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_lerpc(from.raw, to.raw, t, r.raw);
   return r;
@@ -781,7 +886,7 @@ glms_vec3_lerpc(vec3s from, vec3s to, float t) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_mix(vec3s from, vec3s to, float t) {
+glms_vec3_(mix)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_mix(from.raw, to.raw, t, r.raw);
   return r;
@@ -799,7 +904,7 @@ glms_vec3_mix(vec3s from, vec3s to, float t) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_mixc(vec3s from, vec3s to, float t) {
+glms_vec3_(mixc)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_mixc(from.raw, to.raw, t, r.raw);
   return r;
@@ -814,7 +919,7 @@ glms_vec3_mixc(vec3s from, vec3s to, float t) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_step_uni(float edge, vec3s x) {
+glms_vec3_(step_uni)(float edge, vec3s x) {
   vec3s r;
   glm_vec3_step_uni(edge, x.raw, r.raw);
   return r;
@@ -829,7 +934,7 @@ glms_vec3_step_uni(float edge, vec3s x) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_step(vec3s edge, vec3s x) {
+glms_vec3_(step)(vec3s edge, vec3s x) {
   vec3s r;
   glm_vec3_step(edge.raw, x.raw, r.raw);
   return r;
@@ -845,7 +950,7 @@ glms_vec3_step(vec3s edge, vec3s x) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_smoothstep_uni(float edge0, float edge1, vec3s x) {
+glms_vec3_(smoothstep_uni)(float edge0, float edge1, vec3s x) {
   vec3s r;
   glm_vec3_smoothstep_uni(edge0, edge1, x.raw, r.raw);
   return r;
@@ -861,7 +966,7 @@ glms_vec3_smoothstep_uni(float edge0, float edge1, vec3s x) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_smoothstep(vec3s edge0, vec3s edge1, vec3s x) {
+glms_vec3_(smoothstep)(vec3s edge0, vec3s edge1, vec3s x) {
   vec3s r;
   glm_vec3_smoothstep(edge0.raw, edge1.raw, x.raw, r.raw);
   return r;
@@ -879,7 +984,7 @@ glms_vec3_smoothstep(vec3s edge0, vec3s edge1, vec3s x) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_smoothinterp(vec3s from, vec3s to, float t) {
+glms_vec3_(smoothinterp)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_smoothinterp(from.raw, to.raw, t, r.raw);
   return r;
@@ -897,7 +1002,7 @@ glms_vec3_smoothinterp(vec3s from, vec3s to, float t) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_smoothinterpc(vec3s from, vec3s to, float t) {
+glms_vec3_(smoothinterpc)(vec3s from, vec3s to, float t) {
   vec3s r;
   glm_vec3_smoothinterpc(from.raw, to.raw, t, r.raw);
   return r;
@@ -953,7 +1058,7 @@ glms_normalize(vec3s v) {
 /*!
  * @brief swizzle vector components
  *
- * you can use existin masks e.g. GLM_XXX, GLM_ZYX
+ * you can use existing masks e.g. GLM_XXX, GLM_ZYX
  *
  * @param[in]  v    source
  * @param[in]  mask mask
@@ -961,10 +1066,77 @@ glms_normalize(vec3s v) {
  */
 CGLM_INLINE
 vec3s
-glms_vec3_swizzle(vec3s v, int mask) {
+glms_vec3_(swizzle)(vec3s v, int mask) {
   vec3s dest;
   glm_vec3_swizzle(v.raw, mask, dest.raw);
   return dest;
 }
 
+/*!
+ * @brief Create three dimensional vector from pointer
+ *
+ * @param[in]  src  pointer to an array of floats
+ * @returns constructed 3D vector from raw pointer
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(make)(const float * __restrict src) {
+  vec3s dest;
+  glm_vec3_make(src, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief a vector pointing in the same direction as another
+ *
+ * orients a vector to point away from a surface as defined by its normal
+ *
+ * @param[in] n      vector to orient.
+ * @param[in] v      incident vector
+ * @param[in] nref   reference vector
+ * @returns oriented vector, pointing away from the surface.
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(faceforward)(vec3s n, vec3s v, vec3s nref) {
+  vec3s dest;
+  glm_vec3_faceforward(n.raw, v.raw, nref.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief reflection vector using an incident ray and a surface normal
+ *
+ * @param[in]  I    incident vector
+ * @param[in]  N    normalized normal vector
+ * @returns reflection result
+ */
+CGLM_INLINE
+vec3s
+glms_vec3_(reflect)(vec3s v, vec3s n) {
+  vec3s dest;
+  glm_vec3_reflect(v.raw, n.raw, dest.raw);
+  return dest;
+}
+
+/*!
+ * @brief computes refraction vector for an incident vector and a surface normal.
+ *
+ * calculates the refraction vector based on Snell's law. If total internal reflection
+ * occurs (angle too great given eta), dest is set to zero and returns false.
+ * Otherwise, computes refraction vector, stores it in dest, and returns true.
+ *
+ * @param[in]  v    normalized incident vector
+ * @param[in]  n    normalized normal vector
+ * @param[in]  eta  ratio of indices of refraction (incident/transmitted)
+ * @param[out] dest refraction vector if refraction occurs; zero vector otherwise
+ *
+ * @returns true if refraction occurs; false if total internal reflection occurs.
+ */
+CGLM_INLINE
+bool
+glms_vec3_(refract)(vec3s v, vec3s n, float eta, vec3s * __restrict dest) {
+  return glm_vec3_refract(v.raw, n.raw, eta, dest->raw);
+}
+
 #endif /* cglms_vec3s_h */
